local mod = ...


--[[

This seems like a bit of a hack. But it works. The goal was to combine the housing function with an existing workplace part.

You can create a simple assemblage part to combine the original workplace, and a new part that acts as the housing.
The side effect is that you can't see the building's preview when you open the building menu. (Assemblages have this bug.)

To get around this, you have to create an assemblage, then use the Random part constructor with the assemblage as the only "random" part.

]]--




-- Asset -------------------------------------------------------------------------------------
mod:registerAssetId("models/Empty.fbx/Prefab/EmptyPart", "PREFAB_LQ_EMPTY")



-- Comp Building Part -------------------------------------------------------------------------------------
mod:registerEnumValue("BUILDING_PART_TYPE", "HOUSING")
mod:registerEnumValue("ATTACH_NODE_TYPE", "HOUSING")

mod:registerPrefabComponent("models/Empty.fbx/Prefab/EmptyPart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "HOUSING",
})

mod:registerPrefabComponent("models/Empty.fbx/Prefab/EmptyPart/Attach_Housing.001", { DataType = "COMP_BUILDING_ATTACH_NODE", AttachNodeTypeList = { "HOUSING" } })



-- Function to register new housing in workplace -------------------------------------------------------------------------------------------
function registerLQSimple(_part, _capacity, _flagMonument, _conformed)

    if _conformed == true then 
        _prefix = "BUILDING_PART_" 
        _name = "BUILDING_PART_"
    else 
        _prefix = "" 
        _name = "BUILDING_PART_LQ_"
    end

    -- House Setup
    mod:register({
        DataType = "HOUSE_SETUP",
        Id = "HOUSE_SETUP_LQ_" .. _part,
        HouseStatusList = {
            {
                DataType = "HOUSE_STATUS",
                RequiredDesirability = DESIRABILITY_LEVEL.LOW,
                ResourceGenerated = "RESOURCE_HOUSING",
                HouseDensityPartList = { 
                    "BUILDING_PART_LQ_" .. _part,
                }
            },
        },
    })

    mod:register({
        DataType = "BUILDING_FUNCTION_HOUSE",
        Id = "BUILDING_FUNCTION_LQ_" .. _part,
        RelatedZone = "RESIDENTIAL",
        RelatedDesirability = "DESIRABILITY_RESIDENTIAL",
        AssetHouseSetup = "HOUSE_SETUP_LQ_" .. _part,
    })

    -- Assemblage part (allows later combination of functions)
    mod:register({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_LQ_" .. _part .. "_COMBINE",
        Name = "BUILDING_PART_LQ_" .. _part .. "_NAME", 
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
            CoreRandomBuildingPartList = { "PREFAB_LQ_EMPTY" },
            MandatoryBuildingPartList = {
                { BuildingPart = _prefix .. _part },
            },
        },
    })

    -- Random part (combines two functions while still preserving the original function and showing the building in building menu's preview)
    mod:register({
        DataType = "BUILDING_PART",
        Id = "BUILDING_PART_LQ_" .. _part,
        Name = _name .. _part .. "_NAME", 
        Description = "BUILDING_PART_LQ_" .. _part .. "_DESC", 
        ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
            RandomPartList = {
                "BUILDING_PART_LQ_" .. _part .. "_COMBINE",
            }
        },    
        Capacity = _capacity,
        AssetBuildingFunction = "BUILDING_FUNCTION_LQ_" .. _part,
    })

    -- Replace original part with combined workplace and housing part, if this is a simple building (one part)
    if (_flagMonument == false) then
        mod:override({
            Id = "BUILDING_" .. _part,
            AssetCoreBuildingPart = "BUILDING_PART_LQ_" .. _part,
        })
    end

end



-- Run function -------------------------------------------------------------------------------------------
registerLQSimple("BAKERY", 2, false, true)
registerLQSimple("BLACKSMITH", 2, false, true)
registerLQSimple("BREWERY", 3, false, true)
registerLQSimple("CHEESE_MAKER", 2, false, true)
registerLQSimple("COOPERAGE", 3, false, true)
registerLQSimple("DAIRY_FARM", 1, false, true)
registerLQSimple("FISHING_HUT", 1, false, true)
registerLQSimple("FORESTER", 1, false, true)
registerLQSimple("HOP_FARM", 3, false, true)
registerLQSimple("HUNTERS_HUT", 1, false, true)
registerLQSimple("IRON_SMELTER", 2, false, true)
registerLQSimple("SHEEP_FARM", 1, false, true)
registerLQSimple("TAILOR_WORKSHOP", 2, false, true)
registerLQSimple("WEAPONSMITH", 2, false, true)
registerLQSimple("WEAVER_HUT", 2, false, true)
registerLQSimple("WHEAT_FARM", 3, false, true)
registerLQSimple("WINDMILL", 2, false, true)



-- Function to register new housing in monument -------------------------------------------------------------------------------------------
function registerLQList(_monument)

    -- House Setup
    mod:register({
        DataType = "HOUSE_SETUP",
        Id = "HOUSE_SETUP_LQ_" .. _monument,
        HouseStatusList = {
            {
                DataType = "HOUSE_STATUS",
                RequiredDesirability = DESIRABILITY_LEVEL.LOW,
                ResourceGenerated = "RESOURCE_HOUSING",
                HouseDensityPartList = { 
                }
            },
        },
    })

    mod:register({
        DataType = "BUILDING_FUNCTION_HOUSE",
        Id = "BUILDING_FUNCTION_LQ_" .. _monument,
        Name = "BUILDING_FUNCTION_LQ_MONUMENT_NAME",
        RelatedZone = "RESIDENTIAL",
        RelatedDesirability = "DESIRABILITY_RESIDENTIAL",
        AssetHouseSetup = "HOUSE_SETUP_LQ_" .. _monument,
    })

    -- Function list
    mod:override({
        Id = "BUILDING_FUNCTION_LIST_" .. _monument,
        AssetFunctionList = {
            Action = "APPEND",
            "BUILDING_FUNCTION_LQ_" .. _monument,
        },
    })

end

registerLQList("LORD_MANOR")
registerLQList("TAVERN")



-- Mod: Vegetable Farm -------------------------------------------------------------------------------------------
if (foundation.isModLoaded("fe9d3810-8afc-40e5-9fc9-bdbd3421e8cb")) then 
    
    registerLQSimple("VEGETABLE_FARM1", 3, true, true) 

    mod:override({
        Id = "VEGETABLEFARM",
        BuildingPartSetList = {
            {
                Name = "BUILDING_PART_SET_VEGETABLE_FARM1",
                BuildingPartList = { 
                    "BUILDING_PART_LQ_VEGETABLE_FARM1",
                    "BUILDING_PART_VEGETABLE_SHED1",
                }
            },
            {
                Name = "VEGETABLEPATCH_SET1_NAME",
                BuildingPartList = { 
                    "VEGETABLEPATCH2_PART",
                    "VEGETABLEPATCH4_PART",
                    "VEGETABLEPATCH8_PART",
                    "VEGETABLEPATCH6_PART",
                    "VEGETABLEPATCH10_PART",
                    "VEGETABLEPATCH12_PART",
                }
            },
            {
                Name = "VEGETABLEPATCH_SET2_NAME",
                BuildingPartList = { 
                    "VEGETABLEPATCH2N_PART",
                    "VEGETABLEPATCH4N_PART",
                    "VEGETABLEPATCH8N_PART",
                    "VEGETABLEPATCH6N_PART",
                    "VEGETABLEPATCH10N_PART",
                }
            },
            {
                Name = "VEGETABLEPATCH_SET3_NAME",
                BuildingPartList = { 
                    "VEGETABLEPATCH2F_PART",
                    "VEGETABLEPATCH4F_PART",
                    "VEGETABLEPATCH8F_PART",
                    "VEGETABLEPATCH6F_PART",
                    "VEGETABLEPATCH10F_PART",
                }
            },
            {
                Name = "VEGETABLEPATCH_SET4_NAME",
                BuildingPartList = { 
                    "VEGETABLEPATCH2NF_PART",
                    "VEGETABLEPATCH4NF_PART",
                    "VEGETABLEPATCH8NF_PART",
                    "VEGETABLEPATCH6NF_PART",
                    "VEGETABLEPATCH10NF_PART",
                }
            },
            {
                Name = "VEGETABLEPATCH_SET5_NAME",
                BuildingPartList = { 
                    "VEGETABLECART_PART",
                }
            },
        },    
    })

end



-- Mod: Water Mill -------------------------------------------------------------------------------------------
if (foundation.isModLoaded("162964c3-f084-4062-b7d4-7a50487f28ef")) then

    registerLQSimple("WATER_MILL_CORE_FLOUR_PART", 2, true, false) 
    registerLQSimple("WATER_MILL_CORE_TEXTILE_PART", 2, true, false) 
    registerLQSimple("WATER_MILL_CORE_PLANKS_PART", 2, true, false)
    
    mod:override({
        Id = "WATER_MILL_MONUMENT",
        BuildingPartSetList = {
            {
                Name = "PRODUCTION_CORE",
                BuildingPartList = {
                    "BUILDING_PART_LQ_WATER_MILL_CORE_FLOUR_PART",
                    "BUILDING_PART_LQ_WATER_MILL_CORE_TEXTILE_PART",
                    "BUILDING_PART_LQ_WATER_MILL_CORE_PLANKS_PART"
                }
            },
            { 
                Name = "WHEEL",
                BuildingPartList = {
                    "WATER_MILL_EXTENSION_LEFT_BANK_PART",
                    "WATER_MILL_EXTENSION_RIGHT_BANK_PART"
                },
            },
        },
    })

end



-- Mod: Furniture -------------------------------------------------------------------------------------------
if (foundation.isModLoaded("4a8d3ff2-60af-4c75-8950-33e990975862")) then

    registerLQSimple("JOINERWORKSHOP1_PART", 4, true, false)
    registerLQSimple("JOINERWORKSHOP2_PART", 2, true, false)

    mod:override({ Id = "BUILDING_PART_LQ_JOINERWORKSHOP1_PART", Category = "CORE" })
    mod:override({ Id = "BUILDING_PART_LQ_JOINERWORKSHOP2_PART", Category = "CORE" })

    mod:override({
        Id = "JOINERWORKSHOP",
        BuildingPartSetList = {
            {
                Name = "JOINERWORKSHOP_SET1_NAME",
                BuildingPartList = { 
                    "BUILDING_PART_LQ_JOINERWORKSHOP1_PART",
                    "BUILDING_PART_LQ_JOINERWORKSHOP2_PART",
                }
            },
            {
                Name = "JOINERWORKSHOP_SET2_NAME",
                BuildingPartList = { 
                    "JOINERWORKSHOPCOUNTER_PART",
                }
            }
    
        },	
    })

end



-- Mod: Livestock Farm -------------------------------------------------------------------------------------
if (foundation.isModLoaded("debb69c1-eb38-4183-9739-fbafa47568b4")) then

    registerLQSimple("BUTCHERSHOP1_PART", 2, true, false)
    registerLQSimple("LIVESTOCK_CREAMERY", 2, true, true)
	
    mod:override({ Id = "BUILDING_PART_LQ_BUTCHERSHOP1_PART", Category = "CORE" })
    mod:override({ Id = "BUILDING_PART_LQ_LIVESTOCK_CREAMERY", Category = "CORE" })

    mod:override({
        Id = "BUTCHERSHOP",
        AssetCoreBuildingPart = "BUILDING_PART_LQ_BUTCHERSHOP1_PART",
        BuildingPartSetList = {
            {
                Name = "BUTCHERSHOP_SET1_NAME",
                BuildingPartList = { 
                    "BUILDING_PART_LQ_BUTCHERSHOP1_PART",
                }
            },
            {
                Name = "BUTCHERSHOP_SET2_NAME",
                BuildingPartList = { 
                    "BUTCHERSHOP_COUNTER_PART",
                }
            }
        },	
    })

    mod:override({
        Id = "BUILDING_LIVESTOCK_CREAMERY",
        BuildingPartSetList = {
            {
                Name = "BUILDING_SET_LIVESTOCK_CREAMERY_1",
                BuildingPartList = { 
                    "BUILDING_PART_LQ_LIVESTOCK_CREAMERY",
                    "BUILDING_PART_LIVESTOCK_CREAMERY_TABLE",
                }
            }, 
        },	
    })

end



-- Mod: Authority -------------------------------------------------------------------------------------
if (foundation.isModLoaded("93098b10-e659-49db-ae11-f08341972d46")) then

    mod:override({ Id = "BUILDING_PART_AUTHORITY_HALL1", Capacity = 4 })
    mod:override({ Id = "BUILDING_PART_AUTHORITY_HALL2", Capacity = 4 })
    mod:override({ Id = "BUILDING_PART_AUTHORITY_HALL3", Capacity = 4 })
    mod:override({ Id = "BUILDING_PART_AUTHORITY_HALL4", Capacity = 4 })

end



-- Mod: Harbour Trading -------------------------------------------------------------------------------------
if (foundation.isModLoaded("467b9d01-c8f4-42b5-a3a9-5b026376a6a3")) then

    registerLQSimple("HARBORTRADE_HOUSE", 2, true, true)
    registerLQSimple("HARBORTRADE_SHIP1", 6, true, true)
    registerLQSimple("HARBORTRADE_SHIP2", 6, true, true)
    
    mod:override({ Id = "BUILDING_PART_LQ_HARBORTRADE_HOUSE", Category = "CORE" })

    mod:override({
        Id = "BUILDING_HARBORTRADE",
        BuildingPartSetList = {
            {
                Name = "BUILDING_SET_HARBORTRADE1_NAME",
                BuildingPartList = { 
                    "BUILDING_PART_LQ_HARBORTRADE_HOUSE",
                    -- "BUILDING_PART_HARBORTRADE_POLE",
                }
            },
            {
                Name = "BUILDING_SET_HARBORTRADE5_NAME",
                BuildingPartList = { 
                    "BUILDING_PART_LQ_HARBORTRADE_SHIP1",
                    "BUILDING_PART_LQ_HARBORTRADE_SHIP2",
                    "BUILDING_PART_HARBORTRADE_GANGPLANK",
                    "BUILDING_PART_HARBORTRADE_BOAT2",
                    "BUILDING_PART_HARBORTRADE_BOAT3",
                    "BUILDING_PART_HARBORTRADE_BOAT1",
                }
            },
            {
                Name = "BUILDING_SET_HARBORTRADE2_NAME",
                BuildingPartList = { 
                    "BUILDING_PART_HARBORTRADE_WAREHOUSE1",
                    "BUILDING_PART_HARBORTRADE_WAREHOUSE2",
                    "BUILDING_PART_HARBORTRADE_WAREHOUSE3",
                }
            },
            {
                Name = "BUILDING_SET_HARBORTRADE3_NAME",
                BuildingPartList = { 
                    "BUILDING_PART_HARBORTRADE_WAREHOUSE4",
                    "BUILDING_PART_HARBORTRADE_WAREHOUSE5",
                    "BUILDING_PART_HARBORTRADE_WAREHOUSE6",
                }
            },
            {
                Name = "BUILDING_SET_HARBORTRADE4_NAME",
                BuildingPartList = { 
                    "CRANE_PART",
                    "CART_PART",
                    "BARGE_PART",
                    "CRANE_SMALL_PART",
                    "CART_CANOPY_PART",
                    "BARGE_CANOPY_PART",
                    "BARRELS_PART",
                    "BARREL_LARGE_PART",
                    "CRATES_PART",
                    "GRAIN_PART",
                    "SPOOLS_PART",
                    "HAY_PART",
                    "LOGS_PART",
                    "PLANKS_PART",
                    "COAL_PART",
                    "BRICKS_PART",
                    "STONES_PART",
                    "MIX1_PART",
                    "MIX2_PART",
                    "MIX3_PART",
                    "MIX4_PART",
                    "MIX5_PART",
                    "MIX6_PART"
                }
            },
        },
    })
    
end



-- Mod: Plague -------------------------------------------------------------------------------------
if (foundation.isModLoaded("618c6eec-46f0-434d-b645-418462e889d8")) then

    registerLQSimple("PLAGUE_HOSPITAL", 4, true, true)

    mod:override({ Id = "BUILDING_PART_LQ_PLAGUE_HOSPITAL", Category = "CORE" })

    mod:override({
        Id = "BUILDING_PLAGUE_HOSPITAL",
        BuildingPartSetList = {
            {
                Name = "BUILDING_PART_SET_PLAGUE_1",
                BuildingPartList = { 
                    "BUILDING_PART_LQ_PLAGUE_HOSPITAL",
                }
            },
            {
                Name = "BUILDING_PART_SET_PLAGUE_2",
                BuildingPartList = { 
                    "BUILDING_PART_PLAGUE_HOSPITAL_BEDS",
                    "BUILDING_PART_PLAGUE_HOSPITAL_DOOR",
                    "BUILDING_PART_PLAGUE_HOSPITAL_WINDOW",
                }
            },
        },	
    })

end



-- Mod: Fishing Shacks -------------------------------------------------------------------------------------
if (foundation.isModLoaded("a673e06c-5bfd-4325-bd33-398fbd9e5531")) then

    registerLQSimple("JK_FISHING_SHACK_A", 1, true, false)
    registerLQSimple("JK_FISHING_SHACK_B", 1, true, false)
    registerLQSimple("JK_FISHING_SHACK_C", 1, true, false)

    mod:override({
        Id = "JK_FISHING_SHACKS",
        BuildingPartSetList = {
            {
                Name = "JK_FISHING_SHACKS_NAME",
                BuildingPartList = {
                    "BUILDING_PART_LQ_JK_FISHING_SHACK_A",
                    "BUILDING_PART_LQ_JK_FISHING_SHACK_B",
                    "BUILDING_PART_LQ_JK_FISHING_SHACK_C",
                }
            },
            {
                Name = "WOODEN_DOCKS",
                BuildingPartList = {
                    "JK_DOCK_A", "JK_DOCK_B", "JK_DOCK_C", "JK_DOCKS_DOOR_PART",
                }
            },
            {
                Name = "STONE_DOCKS",
                BuildingPartList = {
                    "JK_STONE_DOCK_A",
                    "JK_STONE_DOCK_B",
                }
            },
        },
    })

end



-- Mod: Entertainment -------------------------------------------------------------------------------------
if (foundation.isModLoaded("8d55ee80-e30c-42e3-bf05-1b310d36985f")) then

    registerLQSimple("ENTERTAINMENT_MOD_GALLOW_KEEP_PART", 3, true, false)
    registerLQSimple("ENTERTAINMENT_MOD_BATHHOUSE_PART", 3, true, false)
    registerLQSimple("ENTERTAINMENT_MOD_BROTHEL_PART", 3, true, false)

    mod:override({ Id = "BUILDING_PART_LQ_ENTERTAINMENT_MOD_GALLOW_KEEP_PART", Category = "CORE" })
    mod:override({ Id = "BUILDING_PART_LQ_ENTERTAINMENT_MOD_BROTHEL_PART", Category = "CORE" })
    mod:override({ Id = "BUILDING_PART_LQ_ENTERTAINMENT_MOD_BATHHOUSE_PART", Category = "CORE" })

    mod:override({
        Id = "ENTERTAINMENT_MOD_ENTERTAINMENT_MONUMENT",
        BuildingPartSetList = {
            {
                Name = "ENTERTAINMENT_MOD_ENTERTAINMENT_MONUMENT_GALLOW_KEEP_PART_SET_NAME",
                BuildingPartList = { 
                    "BUILDING_PART_LQ_ENTERTAINMENT_MOD_GALLOW_KEEP_PART",
                    "ENTERTAINMENT_MOD_KEEP_EXTENSION_PART"
                }
            },
            {
                Name = "ENTERTAINMENT_MOD_ENTERTAINMENT_MONUMENT_BROTHEL_PART_SET_NAME",
                BuildingPartList = {
                    "BUILDING_PART_LQ_ENTERTAINMENT_MOD_BROTHEL_PART"
                }
            },
            {
                Name = "ENTERTAINMENT_MOD_ENTERTAINMENT_MONUMENT_BATHHOUSE_PART_SET_NAME",
                BuildingPartList = { 
                    "BUILDING_PART_LQ_ENTERTAINMENT_MOD_BATHHOUSE_PART",
                    "ENTERTAINMENT_MOD_BATH_PART" 
                }
            },
            {
                Name = "ENTERTAINMENT_MOD_ENTERTAINMENT_MONUMENT_BARBERSHOP_PART_SET_NAME",
                BuildingPartList = {
                    "ENTERTAINMENT_MOD_BARBERSHOP_PART"
                }
            },
            {
                Name = "ENTERTAINMENT_MOD_ENTERTAINMENT_MONUMENT_SALOON_PART_SET_NAME",
                BuildingPartList = {
                    "ENTERTAINMENT_MOD_SALOON_PART"
                }
            }
        },
    })

end
